/*******************************************************************

  Copyright (c) 2006, Hyoung-Sun Kim.
  All Rights Reserved.

  You can contact us with
  web site <http://www.voiper.co.kr>
  e-mail <voiper@voiper.co.kr>

  Hyoung-Sun Kim.

  Distributed under the terms of the GNU General Public License

*******************************************************************/

/*

	<SipDemon.c>	2005-11-01,23:44

*/

#include "SipDemon.h"





/* thread objects
*/
SipDemon *newm_SipDemon(IStack *pStack)
{
	SipDemon *tResult = NULL;

	if( pStack==NULL ) return NULL;

	if( (tResult=(SipDemon*)malloc(sizeof(SipDemon)))==NULL )
		return NULL;

	HS_THREAD_INIT(tResult,"SipDemon");

	tResult->mStack = pStack;
	tResult->mNetQ = HS_INVALID_QID;
	return tResult;
}


HS_RESULT deletem_SipDemon(void *pObject)
{
	SipDemon *pObj = (SipDemon*)pObject;

	if( pObj==NULL ) return HS_ERR_NULL_PARAM;

	free(pObj);
	return HS_OK;
}





/* SipDemon Main function
*/
HS_RESULT SipDemon_Main(HS_TQ pQ,void *pObject,void *pArg)
{
	HSQmsg tQmsg;
	HS_RESULT tRet = HS_OK;

	IStack *tStack = NULL;
	SipDemon *pObj = (SipDemon*)pObject;

	HSSrand();

	if( pObj==NULL )
		return HSExitThread(pQ,HS_ERR_NULL_PARAM);
	if( (tStack=pObj->mStack)==NULL )
	{
		deletem_SipDemon(pObj);
		return HSExitThread(pQ,HS_ERR_INVALID_PARAM);
	}

	pObj->mActive = TRUE;
	while(pObj->mActive)
	{
		if( HSThreadGetMessage(pQ,&tQmsg,HS_QM,HS_QM_MAX,TRUE)==HS_OK )
		{
			switch(tQmsg.message)
			{
				case HS_QM_STOP:
					pObj->mActive = FALSE;
					break;
				case HS_QM_CHECK:
					HSPrint("\n%s alive.",pObj->mName);
					break;
				case HS_QM_ADD_UA:
					HSPrint( "\n[+] SIP->ADD_UA" );
					tRet=SipDemonEventAddUa(tStack,(IUa*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_REGIST_UA:
					HSPrint( "\n[+] SIP->REGIST_UA" );
					tRet=SipDemonEventRegistUa(tStack,(HS_UA_HANDLE)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_REMOVE_UA:
					HSPrint( "\n[+] SIP->REMOVE_UA" );
					tRet=SipDemonEventRemoveUa(tStack,(HS_UA_HANDLE)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_MAKE_CALL:
					HSPrint( "\n[+] SIP->MAKE_CALL" );
					tRet=SipDemonEventMakeCall(tStack,(QmMakeCall*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_ACCEPT_CALL:
					HSPrint( "\n[+] SIP->ACCEPT_CALL" );
					tRet=SipDemonEventAcceptCall(tStack,(HS_CALL_HANDLE)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_REMOVE_CALL:
					HSPrint( "\n[+] SIP->REMOVE_CALL" );
					tRet=SipDemonEventRemoveCall(tStack,(SipResponse)(tQmsg.wParam),(HS_CALL_HANDLE)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				/* not implemented yet
				case HS_QM_REMODEL_CALL:
					HSPrint( "\n[+] SIP->REMODEL_CALL" );
					tRet=SipDemonEventRemodelCall(tStack,(QmMakeCall*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				*/
				case HS_QM_SEND_MESSAGE_INFO:
					HSPrint( "\n[+] SIP->MESSAGE_INFO" );
					tRet=SipDemonEventSendMessageINFO(tStack,(QmMessageBody*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_SEND_MESSAGE_MESSAGE:
					HSPrint( "\n[+] SIP->MESSAGE_MESSAGE" );
					tRet=SipDemonEventSendMessageMESSAGE(tStack,(QmMessageBody*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_ENET_MESSAGE:
					HSPrint( "\n[+] SIP->ENET_MESSAGE" );
					tRet=SipDemonEventEnetMessage(tStack,(QmEnetMessage*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_CHANGE_LISTEN_PORT:
					HSPrint( "\n[+] SIP->CHANGE_LISTEN_PORT" );
					tRet=SipDemonEventChangeListenPort(tStack,(HS_USHORT)(tQmsg.wParam),(HS_UINT)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_REMOVE_SOCKET_SET:
					HSPrint( "\n[+] SIP->REMOVE_SOCKET_SET" );
					tRet=SipDemonEventRemoveSocketSet(tStack,(HS_TR_HANDLE)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
				case HS_QM_TIMER:
					HSPrint( "\n[+] SIP->TIMER" );
					tRet=SipDemonEventTimer(tStack,(HSTimerEvent*)(tQmsg.lParam));
					HSPrint( "\n[-] SIP:%u", tRet );
					break;
			}
		}
	}

	delete_IStack(pObj->mStack);
	free(pObj->mStack);
	pObj->mStack = NULL;
	deletem_SipDemon(pObj);
	return HSExitThread(pQ,HS_OK);
}




