/*******************************************************************

  Copyright (c) 2006, Hyoung-Sun Kim.
  All Rights Reserved.

  You can contact us with
  web site <http://www.voiper.co.kr>
  e-mail <voiper@voiper.co.kr>

  Hyoung-Sun Kim.

  Distributed under the terms of the GNU General Public License

*******************************************************************/

/*

	<UaTest.c>		2005-10-22,21:27

*/

#include "SipDefine.h"
#include "UaDefine.h"
#include "SipUI.h"





#define HS_TEST_CHECK_SIP_MESSAGE(tName)					\
if( tMessage==NULL )										\
{															\
	printf( "\n SipMessage Making Error "tName".\n\n" );	\
	return 5;												\
}															\
if( SipMessage_EncodeEx(tMessage,tString,&tPos,1000) != HS_OK )\
{															\
	printf( "\n SipMessage Encoding Error"tName".\n\n" );	\
	return 6;												\
}															\
else														\
	printf("\n\n%s\n\n",tString);							\
free(tMessage);												\
tMessage = NULL



static char *gSdp =
"\
v=0\r\n\
o=alice 2890844526 2890844526 IN IP4 client.atlanta.example.com\r\n\
s=-\r\n\
c=IN IP4 192.0.2.101\r\n\
t=0 0\r\n\
m=audio 49172 RTP/AVP 0 4 8\r\n\
a=rtpmap:0 PCM_U/8000\r\n\
a=rtpmap:4 G.723.1.1/8000\r\n\
m=video 49132 RTP/AVP 31 25 26\r\n\
a=rtpmap:31 H..261/8000\r\n\
a=rtpmap:25 CELB/8000\r\n\
";





int main_make_test();



int main(int argc, char **argv)
{
	if( argc!=2 ) return 0;

	printf( "\n main_sip_ua: %d", main_sip_ua(argv[1]) );

	return 0;
}





int main_make_test()
{
	IStack *tStack = NULL;
	IUa *tUa = NULL;
	IDialogue *tDialogue = NULL;
	ITransaction *tTransaction = NULL;

	ChainUnit *tChain = NULL;
	SipContact *tContact = NULL;
	SipContactUnit *tContactUnit = NULL;
	SipVia *tVia = NULL;

	SdpMessage *tSdp = NULL;
	SipMessage *tMessage = NULL;
	char tString[1024];

	if( (tStack=(IStack*)malloc(sizeof(IStack)))==NULL )
		return 1;
	if( (tUa=(IUa*)malloc(sizeof(IUa)))==NULL )
		return 1;
	if( (tDialogue=(IDialogue*)malloc(sizeof(IDialogue)))==NULL )
		return 1;
	if( (tTransaction=(ITransaction*)malloc(sizeof(ITransaction)))==NULL )
		return 1;

	new_IStack(tStack,"192.168.3.14","test_user_agent");
	new_IUa(tUa,tStack,120);
	new_IDialogue(tDialogue,tUa);
	new_ITransaction(tTransaction,tDialogue,FALSE,TRUE,FALSE,tDialogue->mCseqSeed++);

	/* stack setting
	*/
	tStack->mMaxForward = 50;

	/* ua setting
	*/
	strcpy(tUa->mPassword,"mypass");
	strcpy(tUa->mProxy,"local.proxy.com");
	tUa->mProxyPort = 6050;

	if( (tContact=(SipContact*)malloc(sizeof(SipContact)))==NULL )
	{
		delete_IStack(tStack);				free(tStack);
		return 2;
	}
	new_SipContact(tContact);

	if( (tContactUnit=(SipContactUnit*)malloc(sizeof(SipContactUnit)))==NULL )
	{
		delete_SipContact(tContact);		free(tContact);
		delete_IStack(tStack);				free(tStack);
		return 2;
	}
	new_SipContactUnit(tContactUnit,FALSE);

	tContactUnit->mDisplayName = SipStringAlloc("HSKIM");
	tContactUnit->mUri.mProtocol = SipStringAlloc(tStack->mProtocol);
	tContactUnit->mUri.mUserName = SipStringAlloc("030399466665");
	tContactUnit->mUri.mAddress = SipStringAlloc("local.ua.net");
	tContactUnit->mUri.mPort = SipUintAlloc(5060);

	NoLockList_AttachData(&(tContact->mContactUnits),tContactUnit);
	NoLockList_AttachData(&(tUa->mContacts),tContact);

	/* dialogue setting
	*/
	strcpy(tDialogue->mCallId,"1j9FpLxk3uxtm8tn@sip.mgcp.com");
	strcpy(tDialogue->mLocalTag,"ZZG00_local_876");
	strcpy(tDialogue->mRemoteTag,"ZZG00_remote_876");

	/* transaction setting
	*/
	tTransaction->mBranchId = SipStringAlloc("ZZ00branch876");
	if( (tVia=(SipVia*)malloc(sizeof(SipVia))) != NULL )
	{
		new_SipVia(tVia);
		sprintf(tString,"Via: SIP/%s/%s %s:%u; branch=%s\r\n",
			tStack->mVersion, tTransaction->mIsTcp? "TCP":"UDP", 
			"local.ua.net",5060, "z9hG4bKnashds7"
		);
		SipVia_Decode(tVia, tString, strlen(tString));
		NoLockList_AttachData(&(tTransaction->mVias),tVia);
	}
	tTransaction->mMethod = SipStringAlloc("CONSOL");

	/* add dialogue for call item
	*/
	tDialogue->mDestAddress = SipStringAlloc("remote.proxy.com");
	tDialogue->mDestDisplayName = SipStringAlloc("ICLEE");
	tDialogue->mDestPort = 3030;
	tDialogue->mDestUser = SipStringAlloc("01199261757");


	tMessage = MakeRequestRegister(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("REGISTER");

	tMessage = MakeRequestInvite(tStack,tUa,tDialogue,tTransaction,"application/sdp",gSdp);
	HS_TEST_CHECK_SIP_MESSAGE("INVITE");

	tMessage = MakeRequestAck(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("ACK");

	tMessage = MakeRequestBye(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("BYE");

	tMessage = MakeRequestCancel(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("CANCEL");

	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_Trying);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Trying)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_Ringing);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Ringing)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,"application/sdp",gSdp,e_SipResponse_SessionProgress);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Session Progress)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,"application/sdp",gSdp,e_SipResponse_Ok);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(OK)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_BusyHere);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Busy Here)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_RequestTerminated);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Request Terminated)");

	if( (tDialogue->mVideo=(MediaInfo*)malloc(sizeof(MediaInfo)))!=NULL )
	{
		SdpMessage tRemoteSdp;
		new_SdpMessage(&tRemoteSdp);
		tRemoteSdp.mSessionName = SipStringAlloc("-");

		new_MediaInfo(tDialogue->mVideo);
		tDialogue->mVideo->mLocalRtpAddr = SipStringAlloc("192.168.4.33");
		tDialogue->mVideo->mLocalRtpPort = 5555;
		tDialogue->mVideo->mNumber = (HS_UINT)e_RtpPayloadType_H263;
		tDialogue->mVideo->mRemoteRtpAddr = SipStringAlloc("211.113.114.115");
		tDialogue->mVideo->mRemoteRtpPort = 7777;
		tDialogue->mVideo->mType = SipStringAlloc("video");

		tSdp = IDialogue_MakeSdpAnswer(tDialogue);
#if 0
		if( SdpMessage_Decode(tSdp,gSdp,strlen(gSdp))!=HS_OK )
		{
			printf( "\n SdpMessage Decode Error." );
			return 10;
		}
		else
#endif
		if( SdpMessage_Encode(tSdp,tString,1000)!=HS_OK )
		{
			printf( "\n SdpMessage Encode Error." );
			return 10;
		}
		printf( "\nSDP\n===\n%s", tString );
		delete_SdpMessage(&tRemoteSdp);
		delete_SdpMessage(tSdp);
		free(tSdp);
		tSdp = NULL;
	}

	delete_IStack(tStack);
	free(tStack);
	return 0;
}