/*******************************************************************

  Copyright (c) 2006, Hyoung-Sun Kim.
  All Rights Reserved.

  You can contact us with
  web site <http://www.voiper.co.kr>
  e-mail <voiper@voiper.co.kr>

  Hyoung-Sun Kim.

  Distributed under the terms of the GNU General Public License

*******************************************************************/

/*

	<HSThread.c>	2006-03-20,21:06

*/

#include "HSThread.h"





/*
 *
 * pool of thread
 *
 */
HSSemaphore gQsema;
HS_UINT gQpos = 0;
HS_UINT gQsize = 0;
HS_TQ *gQarray = NULL;





HS_RESULT OpenThreadPool(HS_UINT pSize)
{
	HS_UINT i;

	if( gQsize != 0 || gQarray != NULL ) return HS_ERR;
	if( pSize==0 ) return HS_ERR_INVALID_PARAM;

	if( (gQarray=(HS_TQ*)malloc(sizeof(HS_TQ)*pSize))==NULL )
		return HS_ERR_MALLOC;

	for(i=0; i<pSize; i++)
		gQarray[i] = HS_INVALID_TQ;
	gQsize = pSize;

	HSOpenSemaphore(gQsema);
	return HS_OK;
}


HS_RESULT CloseThreadPool()
{
	if( gQsize==0 || gQarray==NULL ) return HS_ERR;

	HSLockSemaphore(gQsema);
	gQsize = 0;
	free(gQarray);
	gQarray = NULL;
	HSUnlockSemaphore(gQsema);

	HSCloseSemaphore(gQsema);
	return HS_OK;
}


HS_QID AddThreadQ(HS_TQ pQ)
{
	HS_UINT i;

	if( pQ==HS_INVALID_TQ ) return HS_INVALID_QID;
	if( gQsize==0 || gQarray==NULL ) return HS_INVALID_QID;

	HSLockSemaphore(gQsema);
	for(i=0; i<gQsize; i++)
	{
		gQpos++;
		gQpos = gQpos%gQsize;

		if( gQarray[gQpos]==HS_INVALID_TQ )
		{
			gQarray[gQpos] = pQ;
			HSUnlockSemaphore(gQsema);
			return (HS_QID)gQpos;
		}
	}
	HSUnlockSemaphore(gQsema);

	return HS_INVALID_QID;
}


HS_RESULT SubThreadQ(HS_QID pId)
{
	if( pId > gQsize ) return HS_ERR_INVALID_PARAM;
	if( gQsize==0 || gQarray==NULL ) return HS_ERR;

	HSLockSemaphore(gQsema);
	gQarray[pId] = HS_INVALID_TQ;
	HSUnlockSemaphore(gQsema);

	return HS_OK;
}


void ShowThreadPool()
{
	HS_UINT i;

	if( gQsize==0 )
	{
		HSPrint("\ntp> q size is zero");
		return;
	}
	if( gQarray==NULL )
	{
		HSPrint("\ntp> q array is null");
		return;
	}

	HSLockSemaphore(gQsema);
	for(i=0; i<gQsize; i++)
		HSPrint("\ntp> q index[%u]:%08x",i,gQarray[i]);
	HSUnlockSemaphore(gQsema);
}





/*
 *
 * thread functions
 *
 */
typedef struct
{
	HS_TQ mQ;
	void *mObj;
	void *mArg;
	/* typically param1 is mObj pointer,
	         and param2 is mArg pointer.
	*/
	HS_RESULT (*HSThreadDemon)(HS_TQ,void*,void*);
} HSThreadContainer;





#if defined(_WIN32) || defined(_WIN32_WCE)
unsigned __stdcall HSThreadMain(void* pArg)
#elif defined (_VXWORKS)
int HSThreadMain(void *pArg)
#endif
{
	HS_UINT tRetry = 5;
	HSThreadContainer *tContainer = (HSThreadContainer*)pArg;

	/* check parameter container
	*/
	if( tContainer==NULL )
		return 0;
	if( tContainer->mQ==HS_INVALID_TQ ||
		tContainer->mObj==NULL ||
	 	tContainer->HSThreadDemon==NULL )
	{
		free(tContainer);
		return 0;
	}

	while(tRetry--)
	{
		if( tContainer->mQ != HS_INVALID_TQ )
			break;
		HSSleep(10);
	}

	tContainer->HSThreadDemon(tContainer->mQ,tContainer->mObj,tContainer->mArg);
	free(tContainer);
	return 0;
}


HS_QID _HSThreadStart(char *pName,void *pObj,void *pArg,HS_RESULT (*pFunc)(HS_TQ,void*,void*))
{
	HS_TQ tQ;
	HS_QID tQid;
	HSThreadContainer *tContainer = NULL;

	if( pName==NULL || pObj==NULL || pFunc==NULL ) return HS_INVALID_QID;
	if( (tContainer=(HSThreadContainer*)malloc(sizeof(HSThreadContainer)))==NULL )
		return HS_INVALID_QID;

	tContainer->mObj = pObj;
	tContainer->mArg = pArg;
	tContainer->HSThreadDemon = pFunc;
	tContainer->mQ = HS_INVALID_TQ;

#if defined(_WIN32) || defined(_WIN32_WCE)
	CreateThread(NULL,0,&HSThreadMain,tContainer,0,&(tContainer->mQ));
	tQ = tContainer->mQ;
	if( (tQid=AddThreadQ(tQ))==HS_INVALID_QID )
	{
		PostThreadMessage(tQ,HS_QM_STOP,(WPARAM)0,(LPARAM)NULL);
		return HS_INVALID_QID;
	}
#elif defined(_VXWORKS)
	if( (tContainer->mQ=msgQCreate(100,sizeof(HSQmsg),MSG_Q_FIFO))==NULL )
	{
		free(tContainer);
		return HS_INVALID_QID;
	}

	tQ = tContainer->mQ;
	if( (tQid=AddThreadQ(tQ))==HS_INVALID_QID )
	{
		msgQDelete(tQ);
		free(tContainer);
		return HS_INVALID_QID;
	}
	taskSpawn(pName,120,0,16384,HSThreadMain,(int)tContainer,0,0,0,0,0,0,0,0,0);
#endif

	return tQid;
}


HS_RESULT _HSThreadStop(HS_QID pQid)
{
	HS_RESULT tRet = HS_OK;

	if( pQid==HS_INVALID_QID )
		return HS_ERR_INVALID_PARAM;

	tRet = _HSThreadSendMessage(pQid,HS_QM_STOP,0,0);

	SubThreadQ(pQid);
	return tRet;
}


HS_RESULT HSExitThread(HS_TQ pQ,HS_RESULT pResult)
{
#if defined(_VXWORKS)
	if( pQ != HS_INVALID_TQ )
		msgQDelete(pQ);
#endif

	return pResult;
}


HS_RESULT _HSThreadSendMessage(HS_QID pQid,HS_UINT pMessage,HS_UINT pWparam,HS_UINT pLparam)
{
	HS_TQ tQ = HS_INVALID_TQ;
#if defined(_VXWORKS)
	HSQmsg tQmsg;
#endif

	if( pQid==HS_INVALID_QID ) return HS_ERR_INVALID_PARAM;

	HSLockSemaphore(gQsema);
	if( pQid < gQsize )
		tQ = gQarray[pQid];
	else
		tQ = HS_INVALID_TQ;
	HSUnlockSemaphore(gQsema);

	if( tQ==HS_INVALID_TQ ) return HS_ERR;

#if defined(_WIN32) || defined(_WIN32_WCE)
	PostThreadMessage(tQ,pMessage,(WPARAM)pWparam,(LPARAM)pLparam);
#elif defined(_VXWORKS)
	tQmsg.message = pMessage;
	tQmsg.wParam  = pWparam;
	tQmsg.lParam  = pLparam;
	msgQSend(tQ,(char*)&tQmsg,sizeof(tQmsg),NO_WAIT,MSG_PRI_NORMAL);
#endif

	return HS_OK;
}


HS_RESULT HSThreadGetMessage(HS_TQ pQ,HSQmsg *pQmsg,HS_UINT pMin,HS_UINT pMax,BOOL pIsWait)
{
	if( pQ==HS_INVALID_TQ || pQmsg==NULL )
		return HS_ERR_NULL_PARAM;

	/* wait
	*/
	if( pIsWait==TRUE )
	{
#if defined(_WIN32) || defined(_WIN32_WCE)
		if( GetMessage(pQmsg,NULL,pMin,pMax) )
#elif defined (_VXWORKS)
		if( msgQReceive(pQ,(char*)pQmsg,sizeof(HSQmsg),WAIT_FOREVER) != ERROR/*-1*/ )
#endif
			return HS_OK;
		else
			return HS_ERR;
	}

	/* no wait
	*/
#if defined(_WIN32) || defined(_WIN32_WCE)
	if( PeekMessage(pQmsg,NULL,pMin,pMax,PM_REMOVE) )
#elif defined (_VXWORKS)
	if( msgQReceive(pQ,(char*)pQmsg,sizeof(HSQmsg),NO_WAIT) != ERROR/*-1*/ )
#endif
		return HS_OK;
	else
		return HS_ERR;
}






